/**

   DicoPro project
   XMLTrans : an XML transducer

   Dominique Petitpierre, ISSCO, 1999
  
   Copyright (c) 1999,2000 ISSCO/University of Geneva, Geneva, Switzerland

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

import org.w3c.dom.NodeList;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.Locator;
import com.sun.xml.tree.XmlWritable;
import com.sun.xml.tree.ParseContext;
import com.sun.xml.tree.ElementNode;

import ch.unige.issco.xmltrans.util.*;
import ch.unige.issco.xmltrans.parser.*;

public class TransduceElement extends ElementNode 
{
    private static final java.lang.String rcsid
	= "@(#)$Id: TransduceElement.java,v 1.18 1999/07/29 12:58:56 petitp Exp $";

    public static ASTRuleSet startRuleSet;

    public void startParse(ParseContext context) {

	if (Debug.test(Debug.XMLPARSE)) {
	    Debug.msg(contextMsg(context) + ": <" + getNodeName() + ">"
		      + " starting parse");
	}
	return;
    }

    public void doneParse(ParseContext context)
    {

	ASTRuleSet ruleSet;
	Symbol startEltSym;
	NodeList childNodes;
	int i;
	int nChild;


	startEltSym = (Symbol) XMLTrans.root.startElements.get(getNodeName());
	if (startEltSym != null && startEltSym.value != null) {
	    ruleSet =(ASTRuleSet) ((Symbol) startEltSym.value).value;
	}
	else {
	    ruleSet = null;
	}
	if (ruleSet == null ) {
	    // should not happen
	    // TODO: turn this into an exception
	    System.err.println(contextMsg(context) + ": <" + getNodeName()
			       + "> no rule set for this start element");
	} else {
	    startRuleSet = ruleSet;
	    childNodes = getParentNode().getChildNodes();
	    // dirty hack: should have a match for a single root element
	    // (not sure that it is the last child...)
	    for (i = childNodes.getLength() - 1 ;
		 childNodes.item(i) != this;
		 i--);
	    if (Debug.test(Debug.MATCHING|Debug.MATCHED)) {
		Debug.msg(contextMsg(context) + ": <" + getNodeName() + ">"
			  + " trying rule set @" + ruleSet.set.name);
	    }
	    if (! ruleSet.match(childNodes, i, i+1, XMLTrans.doc,
				XMLTrans.transduced)) {
		// match failed or not spanning
		// TODO: turn this into an exception
		String msg = (" <" + getNodeName() + ">"
			      + " did not match any rule in rule set @"
			      + ruleSet.set.name);
		System.err.println( contextMsg(context) + ":" + msg);
		XMLTrans.transduced
		    .appendChild(XMLTrans.doc.createComment(msg + " "));
	    }
	    // spit out the result!
	    try {
		childNodes = XMLTrans.transduced.getChildNodes();
		nChild = childNodes.getLength();
		for (i=0; i < nChild; i++) {
		    ((XmlWritable) childNodes.item(i)
		     ).writeXml(XMLTrans.writeContext);
		    if (Debug.test(Debug.ALL)) {
			XMLTrans.outputstream.flush();
		    }
		}
	    }
	    catch (java.io.IOException e) {
		System.err.println(contextMsg(context) + ": <"
				   + getNodeName() + ">"
				   + "I/O error while printing transduced"
				   + " result:\n" + e.getMessage());
	    }
	}
	// detach ourself from tree in order to save space
	getParentNode().removeChild(this);
	// clean up the built document
	childNodes = XMLTrans.transduced.getChildNodes();
	for (i = childNodes.getLength() - 1; i >= 0; i--) {
	    XMLTrans.transduced.removeChild(childNodes.item(i));
	}
	return;
    }

    String contextMsg (ParseContext context)
    {
	Locator locator = context.getLocator();
	int col = locator.getColumnNumber();
	String cwd;
	String url;
	
	// remove the "file:" prefix as well as the current directory
	// from the systemId
	cwd = System.getProperty("user.dir") + "/";
	url = locator.getSystemId();
	if ( url.regionMatches(0,"file:",0,"file:".length())) {
	    url = url.substring("file:".length());
	}
	if (url.regionMatches(0,cwd,0,cwd.length())) {
	    url = url.substring(cwd.length());
	}
	return ( url + ":" + locator.getLineNumber()
		 + (col <= 0 ? "" : " column " + col )
		 );
    }
}
