/**

   DicoPro project
   XMLTrans : an XML transducer

   Dominique Petitpierre, ISSCO, 1999
  
   Copyright (c) 1999,2000 ISSCO/University of Geneva, Geneva, Switzerland

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/


import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import org.xml.sax.helpers.ParserFactory;

import com.sun.xml.tree.*;
import com.sun.xml.parser.Resolver;

import ch.unige.issco.xmltrans.*;
import ch.unige.issco.xmltrans.util.*;
import ch.unige.issco.xmltrans.parser.*;

public class XMLTrans {

  private static final java.lang.String rcsid
    = "@(#)$Id: XMLTrans.java,v 1.16 1999/07/29 18:26:17 petitp Exp $";


  public static ASTGrammar root;
  public static PrintWriter outputstream;
  public static XmlDocument doc;
  public static XmlWriteContext writeContext;
    // public static DocumentFragment transduced;
    public static ElementNode transduced;


  public static void main(String args[]) {
    XMLTransParser xtrparser;
    InputSource input;
    FileInputStream grammarstream;
    XmlDocumentBuilder	xmlbuilder;
    Parser		xmlparser;

    String ProgramName = "XMLTrans";
    String ProgramVersion = "0.3";

    try {
	// TODO: set debug bits in a more symbolic way
      Debug.set(Integer.getInteger("debugBits",Debug.NONE).intValue());
      if (args.length < 1 || args.length > 3) {
	  throw new XMLTransException("Usage:"
				      + " java" + ProgramName
				      + " grammarfile"
				      + " [ inputfile [ outputfile ] ]" );
      }
      try {
	grammarstream = new FileInputStream(args[0]);
    } catch (FileNotFoundException e) {
	throw new XMLTransException("Grammar file " + args[0]
				    + " not found.");
    }
    if (args.length >= 2) {
	try {
	    // Turn the filename into an input source
	    //
	    // NOTE:  The input source must have a "system ID" if
	    // there are relative URLs in the input document.  The
	    // static resolver methods handle that automatically
	    // in most cases.
	    input = Resolver.createInputSource (new File (args [1]));
	} catch (FileNotFoundException e) {
	    throw new XMLTransException("Input file " + args[1]
					+ " not found.");
	} catch (IOException e1) {
	    throw new XMLTransException("Input file " + args[1]
					+ " I/O exception.");
	}
    } else {
	try {
	    input = Resolver.createInputSource("text/xml;charset=iso8859-1",
				       System.in, false, "file");
	} catch (IOException e1) {
	    throw new XMLTransException("standard input I/O exception.");
	}
    }
    if (args.length >= 3) {
	try {
	    outputstream = new PrintWriter(new FileWriter(args[2]));
	} catch (FileNotFoundException e) {
	    throw new XMLTransException("Output file " + args[2]
					+ " not found.");
	} catch (IOException e1) {
	    throw new XMLTransException("Output file " + args[2]
					+ " I/O exception.");
	}
    } else {
	outputstream = new PrintWriter(System.out);
    }
    try {
      xtrparser = new XMLTransParser(grammarstream);
      // root = (SimpleNode) xtrparser.jjtree.rootNode();
      root = xtrparser.Grammar();
      grammarstream.close();
      if (Debug.test(Debug.PARSED)) {
	  root.dump(">");
      }
      if (XMLTransParser.parseErrors) {
	  throw new XMLTransException("unrecoverable errors");
      }
      else {
	  System.err.println(ProgramName
			     + ":  Grammar Parsing completed successfully.");
      }
    } catch (ParseException e) {
	throw new XMLTransException("Encountered errors during parse.",e);
    } catch (IOException e1) {
      throw new XMLTransException("I/O exception.",e1);
    }
    try {
	doc = new XmlDocument (); // target document to hold transduced element
	if (Debug.test(Debug.PARSED)) {
	    Debug.msg("docid=" + root.docId
		      + " publicid=" + root.publicId
		      +" systemid=" + root.systemId
		      +" internalsubset=" + root.internalSubset);
	}
	DocumentType docType = doc.setDoctype(root.publicId, root.systemId, root.internalSubset);
	doc.setSystemId(root.systemId);
	writeContext = doc.createWriteContext(outputstream, 0);
	if (Debug.test(Debug.PARSED) && docType != null) {
	    
	    
	    Debug.msg("System id =" + doc.getSystemId()
		      + " DTD name = " + docType.getName()
		      + " equal" + (doc.getDoctype() ==  docType ? "OK": "KO")
		      + " entities:");
	    NamedNodeMap e = docType.getEntities();
	    if ( e != null) {
		for (int i = e.getLength() - 1; i >=0; i--) {
		    Debug.msg("publicid=" + ((Entity) e.item(i)).getPublicId()
			      + " systemid=" + ((Entity) e.item(i)).getSystemId()
			      + "entity=" + ((Entity) e.item(i)).getNodeName()
			      + "notation=" + ((Entity) e.item(i)).getNotationName() );
		}
	    }
	}
	//	transduced = (DocumentFragment) doc.createDocumentFragment();
	transduced = (ElementNode) doc.createElement((root.docId==null?"root":root.docId));
	doc.appendChild(transduced);
	xmlbuilder = new XmlDocumentBuilder ();
	String ignoreWhiteSpace = System.getProperty("ignoreWhiteSpace",
						     "true");
	xmlbuilder.setIgnoringLexicalInfo(Boolean.valueOf(ignoreWhiteSpace)
					 .booleanValue());
	if (Debug.test(Debug.XMLPARSE)) {
	    Debug.msg("XML ignorable white space ignored: "
		      + xmlbuilder.isIgnoringLexicalInfo());
	}
	xmlparser = ParserFactory.makeParser ();
	{
	    Properties              props;
	    SimpleElementFactory    factory;
 
	    props = new Properties ();
	    props.put ("*Element","com.sun.xml.tree.ElementNode");
	    for (Enumeration e = root.startElements.elements();
		 e.hasMoreElements();
		 props.put (((Symbol) e.nextElement()).name,
			    "TransduceElement"));
	    factory = new SimpleElementFactory ();
	    factory.addMapping (props, XMLTrans.class.getClassLoader ());
	    xmlbuilder.setElementFactory (factory);
	}
	xmlparser.setDocumentHandler (xmlbuilder);
	xmlparser.setErrorHandler (new XMLTransErrorHandler ());

	// transduce
	xmlparser.parse (input);

	if (Debug.test(Debug.XMLPARSE)) {
	    // checks what hasn't been transduced
	    Element topElement;
	    topElement = xmlbuilder.getDocument ().getDocumentElement ();
	    if (topElement != null) { // have all elements been removed?
		topElement.normalize ();
	    }
	    Debug.msg("XML source parts for which no transduction was tried:");
	    xmlbuilder.getDocument ().write (System.err);
	}
    } catch (SAXParseException err) {
	/*
	  Exception	x = err;
	  if (err.getException () != null)
	  x = err.getException ();
	  x.printStackTrace ();
	*/
	throw new XMLTransException("** Parsing error" 
				    + ", line " + err.getLineNumber ()
				    + ", uri " + err.getSystemId () + "\n"
				    + "   " + err.getMessage (),
				    err);
    } catch (SAXException e) {
	Exception	x = e;
	if (e.getException () != null)
	    x = e.getException ();
	throw new XMLTransException("",x);
    } catch (Exception t) {
	throw new XMLTransException("",t);
    }
    try {
	outputstream.close();
	if ( input.getByteStream() != null ) {
	    input.getByteStream().close();
	} else if (input.getCharacterStream() != null ) {
	    input.getCharacterStream().close();
	}
    } catch (IOException e) {
	throw new XMLTransException("I/O exception during close");
    }
  }
  catch (XMLTransException e) {
      System.err.println(ProgramName + " " + ProgramVersion + " fatal error:\n"
			 + e.getMessage());
      if (e.causingException != null) {
	  e.causingException.printStackTrace ();
      }
      System.exit(1);
  }
  System.exit(0);
  }
}
