/**

   DicoPro project
   XMLTrans : an XML transducer

   Dominique Petitpierre, ISSCO, 1999
  
   Copyright (c) 1999,2000 ISSCO/University of Geneva, Geneva, Switzerland

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
*/

package ch.unige.issco.xmltrans.parser;

import org.w3c.dom.NodeList;
import org.w3c.dom.Node;


public class PatternNode extends SimpleNode {

    private static final java.lang.String rcsid
	= "@(#)$Id: PatternNode.java,v 1.9 1999/04/07 01:19:28 petitp Exp $";


    public PatternNode nextPat;
    public PatternNode upperPat;

    public PatternNode(int id) {
	super(id);
    }
 
    public PatternNode(XMLTransParser p, int id) {
	super(p, id);
    }

    // to be overriden in extensions
    public boolean match(Node node) {
	return (node != null);
    }

    // TODO: make this an interface,
    // generalize the matchRest
    // to be overriden in extensions
    // This version is appropriate for "leaf" patterns that match exactly
    // one node
    public boolean match(NodeList nodeList,
		     int startPos,
		     int endPos)
    {
	PatternNode contPat;
	int matchPos;

	if ( ! match(nodeList.item(startPos)) ) {
	    return false; // mismatch, wrong node type, null node
	}
	matchPos = startPos + 1;
	if (nextPat != null) {
	    // advance in the chain
	    return nextPat.match(nodeList, matchPos, endPos);
	}
	else if (upperPat != null) {
	    return ((ASTRegPat) upperPat).matchRest(nodeList, matchPos, endPos);
	}
	else {
	    // assertion: not possible
	    // TODO: turn this into an exception
	    if (Debug.test(Debug.ASSERTION)) {
		Debug.msg("internal error: simple pattern parent is not"
			  + " a RegPat");
	    }
	    return false;
	}
    }

    public void assignNextPat()
    {
    }
}
