/**
   DicoPro project
   XMLTrans : an XML transducer

   Dominique Petitpierre, ISSCO, 1999
  
   Copyright (c) 1999,2000 ISSCO/University of Geneva, Geneva, Switzerland

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

package ch.unige.issco.xmltrans.util;

import java.util.Enumeration;
import java.util.NoSuchElementException;

public class Chain
{
    private static final java.lang.String rcsid
	= "@(#)$Id: Chain.java,v 1.3 1999/04/02 04:28:36 petitp Exp $";

    public Chain next;
    public Object item;

    public Chain()
    {
	next = null;  // indicates empty chain
    }

    public Chain(Object obj)
    {
	item = obj;
	next = this;  // chain of one element
    }

    public Chain insert(Object obj)
    {
	Chain newLink;

	if (next == null) { // empty chain
	    next = this; // chain of one element 
	    item = obj;
	    return this;
	}
	newLink = new Chain();
	newLink.item = obj;
	newLink.next = next;
	next = newLink;
	// newLink is the last link of the chain
	// newLink.next indicates the original start of the chain
	// "this" is the before last link of the chain (>1)
	return newLink;
    }

    public Object peek()
    {
	if (next != null) {
	    return next.item;
	}
	throw new NoSuchElementException("Chain");
    }

    public Object pop()
    {
	Chain top;

	if (next != null) {
	    top = next;
	    // severe the top link;
	    next = next.next;
	    top.next = null;
	    return top.item;
	}
	throw new NoSuchElementException("Chain");
    }

    boolean isEmpty()
    {
	return (next == null);
    }

    // cut the tail of a chain
    // The chain cannot be empty (-> NullPointer Exception)
    public Chain trim(Chain newLastLink)
    {
	// assertion: newLastLink is part of this chain, in particular
	// it could be that newLastLink == this
	newLastLink.next = next;
	// severe the dangling tail.
	// Also: empties a chain if newLastLink == this
	next = null;
	return newLastLink;
    }

    void zero()
    {
	next = null;  // same effect as trim(this)
    }

    public ChainEnumerator elements()
    {
	return new ChainEnumerator(this);
    }

    public static final void main(String[] args)
    {
	Chain chain;
	Enumeration e;

	chain = new Chain("a");
	chain.insert("b");
	chain.insert("c");
	for (e = chain.elements(); e.hasMoreElements();
	     System.out.print(e.nextElement()));
	System.out.println();

	chain.trim(chain); // empties the chain
	chain = chain.insert("a");
	chain = chain.insert("b");
	chain = chain.insert("c");
	for (e = chain.elements(); e.hasMoreElements();
	     System.out.print(e.nextElement()));
	System.out.println();
    }
}

class ChainEnumerator implements Enumeration
{
    Chain currentLink;
    Chain endLink;

    ChainEnumerator(Chain chain)
    {
	currentLink = chain;
	endLink = chain.next;
    }

    public boolean hasMoreElements()
    {
	return (endLink != null);
    }
 
    public Object nextElement()
    {
        if (endLink != null) {
	    currentLink = currentLink.next;
	    if (currentLink.next == endLink) {
		endLink = null;
	    }
	    return currentLink.item;
	}
	throw new NoSuchElementException("ChainEnumerator");
    }
}
