/**

   DicoPro project
   XMLTrans : an XML transducer

   Dominique Petitpierre, ISSCO, 1999
  
   Copyright (c) 1999,2000 ISSCO/University of Geneva, Geneva, Switzerland

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
*/

package ch.unige.issco.xmltrans.util;

public class EscapeChar {

  private static final java.lang.String rcsid
    = "@(#)$Id: EscapeChar.java,v 1.4 1999/04/07 15:22:55 petitp Exp $";


  public static String decode(String encoded) {
    StringBuffer decoded = new StringBuffer();
    int i=0;
    int j;
    int radix;
    int lastdigit;
    int length;
    String charCode;
    char currentChar;
    char c;

    length = encoded.length();
    while(i < length) {
      currentChar = encoded.charAt(i);
      i++;
      if (currentChar != '\\'
	  || i >= length ) {  // really should throw an exception
        decoded.append(currentChar);
      } else {
	  currentChar = encoded.charAt(i);
	  i++;
	  switch (currentChar) {
	  case 'n': c = '\n'; break; 
	  case 't': c = '\t'; break; 
	  case 'b': c = '\b'; break; 
	  case 'r': c = '\r'; break; 
	  case 'f': c = '\f'; break;
	  case '0':
	  case '1':
	  case '2':
	  case '3':
	      if ( currentChar == '0'
		   && i < length
		   && ( encoded.charAt(i) == 'x'
			|| encoded.charAt(i) == 'X')) {
		  radix = 16;
		  lastdigit = i + 2; // 2 digits max
		  i++; // i points to first significant digit
	      } else {
		  radix = 8 ;
		  lastdigit = i + 1 ; // 3 digits max
		  i--; // i points to first significant digit
	      }
	      j = i;
	      while (j < length
		     && j <= lastdigit
		     && Character.digit(encoded.charAt(j),radix) >= 0 ) {
		  j++;
	      }
	      c = (char) Integer.parseInt(encoded.substring(i,j),radix);
	      i = j;
	      break;
	  case 'u':
	  case 'U':
	      radix = 16;
	      lastdigit = i + 3; // 4 digits max
	      j = i;
	      while (j < length
		     && j <= lastdigit
		     && Character.digit(encoded.charAt(j),radix) >= 0 ) {
		  j++;
	      }
	      // if i == j, should throw an exception
	      c = (char) Integer.parseInt(encoded.substring(i,j),radix);
	      i = j;
	      break;
	  default:
	      c = currentChar;
	      break;
	  }
	  decoded.append(c);
      }
    }
    return(decoded.toString());
  }

 /*
   encode will convert a string such that non printable characters will
   be escaped. The delimiter, if found in the string will also be escaped.
   Should try to use MessageFormat to abide to Locale conventions.
 */
  public static String encode(String str, char delimiter) {
    StringBuffer encoded = new StringBuffer();
    char ch;
    int length;
    char currentChar;
    String prefix;
    String template;
    String s;

    if (str == null) {
	return null;
    }
    length = str.length();
    for (int i = 0; i < length; i++) {
	currentChar = str.charAt(i);
        switch (currentChar) {
	case '\b':
	    encoded.append("\\b");
	    continue;
	case '\t':
	    encoded.append("\\t");
	    continue;
	case '\n':
	    encoded.append("\\n");
	    continue;
	case '\f':
	    encoded.append("\\f");
	    continue;
	case '\r':
	    encoded.append("\\r");
	    continue;
	case '\\':
	    encoded.append("\\\\");
	    continue;
	default:
	    if (currentChar == delimiter ) {
		encoded.append("\\");
		encoded.append(delimiter);
	    } else if (( currentChar >= 0x20  // really should be isprintable()
			 && currentChar <= 0x7e)
		       || ( currentChar >= 0xa0
			    && currentChar <= 0xff)) {
		encoded.append(currentChar);
	    } else {
		if (currentChar > 0xff ) {
		    prefix = "\\u";
		    template = "0000";
		} else {
		    prefix = "\\0x";
		    template = "00";
		}
		s = template + Integer.toHexString(currentChar);
		encoded.append(prefix
			       + s.substring(s.length() - template.length(),
					     s.length()));
	    }
	    continue;
        }
    }
    return encoded.toString();
  }
 

  public static void main(String[] args) {
    System.out.println(decode(args[0]));
    System.out.println("\"" + encode(decode(args[0]),'"') + "\"");
  }
  
}
